/*
 * File......: DELAYRUN.PRG
 * Author....: Berend M. Tober
 * CIS ID....: 70541,1030
 * Date......: $Date$
 * Revision..: $Revision$
 * Log file..: $Logfile$
 * 
 * This is an original work by Berend M. Tober and is placed in the
 * public domain.
 *
 * Modification history:
 * ---------------------
 *
 * $Log$
 *
 */

/*  $DOC$
 *  $FUNCNAME$
 *     FT_DELAYRUN()
 *  $CATEGORY$
 *     To be assigned
 *  $ONELINER$
 *     Evaluates a codeblock on a deferred schedule.
 *  $SYNTAX$
 *     FT_DELAYRUN( <bRoutine>, <cTitle>, <dDate>, <cTime> ) -> xRetVal
 *  $ARGUMENTS$
 *     <bRoutine> - Codeblock to evaluate.
 *     <cTitle>   - Title to display while waiting for evaluation.
 *     <dDate>    - Date at which evaluation is to begin.  Defaults to
 *                  tomorrow.
 *     <cTime>    - Time at which evaluation is to begin.  Defaults to
 *                  a time between midnight and 1:00am.
 *  $RETURNS$
 *     <xRetVal>  - Result of code block.
 *  $DESCRIPTION$
 *     |--------------------------------------------------------------|
 *     This module is useful if you want to write applications that
 *     offer the option to do any tasks, such as printing reports or
 *     REINDEXing or PACKing of .dbf files, on a deferred schedule so
 *     as to minimize inconvenience to the user community.
 *
 *     In your application you would write a call to YOUR routine as a
 *     codeblock and pass that code block to FT_DELAYRUN as the first
 *     parameter.  FT_DELAYRUN then blanks the screen, and displays the
 *     scheduling status in a randomly-positioned box that moves every
 *     five seconds until the trigger date/time has been reached.  Thus
 *     it behaves as a 'screen saver' while waiting.
 *
 *     At the trigger time your codeblock is evaluated, and whatever
 *     your codeblock returns is returned by FT_DELAYRUN.
 *
 *     The trigger date/time defaults to some time in the first hour
 *     of the next day (i.e., the first hour following the next-
 *     occuring midnight).  The default tigger time will be as many
 *     minutes after midnight as the number of minutes after the hour
 *     when FT_DELAYRUN is called.
 *
 *     The deferral schedule can be interrupted by cancelling with the
 *     ESC key, by initiating an edit of the trigger date/time using
 *     the ENTER key, or by forcing immediate execution with
 *     CTRL-ENTER.
 *
 *     Command-style call is defined in DELAYRUN.CH file.
 *  $EXAMPLES$
 *
 *     // As a function call
 *     lFinished := FT_DELAYRUN( bRoutine )
 *
 *     // As a command defined in DELAYRUN.CH
 *     DELAYRUN BLOCK bRoutine TITLE "Waiting for a surprize!!!" TO lFinished
 *  $SEEALSO$
 *  $INCLUDE$
 *     box.ch
 *     inkey.ch
 *     common.ch
 *     delayrun.ch
 *  $END$
 *
 */

#include "box.ch"
#include "inkey.ch"
#include "common.ch"

/*
// File:    DELAYRUN.CH   // Command for FT_DELAYRUN()
// Author:  Berend M. Tober
// Date:    1994/07/12
#ifndef _DELAYRUN_CH

   #xcommand DELAYRUN BLOCK <b> [TITLE <c>] [XDATE <d>] [XTIME <t>] TO <r>;
   => <r> := FT_DELAYRUN(  <b>, <c>, <d>, <t>)

   #xcommand DELAYRUN BLOCK <b> [TITLE <c>] [XDATE <d>] [XTIME <t>];
   => FT_DELAYRUN(  <b>, <c>, <d>, <t>)

   #define _DELAYRUN_CH

#endif
*/

STATIC dDate, cTime  // Need to have file-wide scope for GETs to work
                     // across both subroutines

#ifdef FT_TEST

#include "delayrun.ch"

PROCEDURE T_DELAYRUN    // Sample program
   LOCAL bRoutine

   CLEAR SCREEN

   * Example 1: Demonstrates call to codeblock with return value

   ALERT("Press any key to begin example #1")
   bRoutine :={|| ALERT("This is the surprize?"), "#1 Complete..." }

   @ 0, 0 SAY FT_DELAYRUN( bRoutine, "Waiting for a surprize!!!" )

   * Example 2: Demonstrates call to codeblock that calls UDF.

   ALERT("Press any key to begin example #2")
   bRoutine :={|| DUMMYPACK()}
   CLEAR SCREEN

   FT_DELAYRUN( bRoutine,"Pack databases, or whatever" )

   * Example 3: Demonstrates command with return value

   ALERT("Press any key to begin example #3")
   bRoutine := {|| ALERT("Another surprize?"), "#1 Complete..." }

   DELAYRUN BLOCK bRoutine TITLE "Waiting for another surprize!!!" TO bRoutine
   @ 0, 0 SAY bRoutine

   * Example 4: Demonstrates call command that calls UDF.

   ALERT("Press any key to begin example #4")
   bRoutine :={|| DUMMYPACK()}
   CLEAR SCREEN

   DELAYRUN BLOCK bRoutine TITLE "This is the final example"

   CLEAR SCREEN
   ? "End of demonstration"

   RETURN

FUNCTION DUMMYPACK()
   * This might be your function to PACK dbf's
   ALERT("Whatever functions you want appear here ...")
   RETURN NIL
#endif

#ifdef STANDALONE
/*
   In principal, this section of code could be used to execute
   any DOS command or program from the DOS prompt on a deferred
   schedule, however the host computer must have sufficient
   memory available to contain both this program and the target
   application
*/

PROCEDURE DELAYRUN()
   * Stand-alone executable

   PARAMETERS cRoutine

   IF EMPTY( cRoutine )
      CLEAR SCREEN
      ? "Program.....: DELAYRUN"
      ? "Author......: Berend M. Tober"
      ? "Date........: 1994/07/13"
      ? "CIS id......: 70541,1030"
      ?
      ? "Syntax: DELAYRUN <program>"
      ?
      QUIT
   ENDIF

   FT_DELAYRUN( {||Execute(cRoutine)}, cRoutine )
   RETURN

***************************** Execute( cRoutine ) *****************************
STATIC FUNCTION Execute( cRoutine )
   RUN (cRoutine)
   RETURN NIL
*  end of Execute( cRoutine )

#endif

********************************* FT_DELAYRUN() *********************************
FUNCTION FT_DELAYRUN( bRoutine, cTitle, uDate, uTime )
   * Implements a delayed execution of a codeblock
   LOCAL cScreen  := SAVESCREEN()
   LOCAL cColor   := SETCOLOR("W/N", "N/W")
   LOCAL nKey    
   LOCAL nWait    := 0.1   // On first call, don't (hardly) wait
   LOCAL xRetVal
   MEMVAR GetList

   DEFAULT bRoutine  TO {|| TRUE }           // No action
   DEFAULT cTitle    TO ""                  // No title
   DEFAULT uDate     TO (1+DATE())           // Packing will occur tomorrow,
   DEFAULT uTime     TO "00"+RIGHT(TIME(),6) // after midnight

   dDate := uDate
   cTime := uTime

   DO WHILE (nKey := INKEY(nWait)) != K_ESC
      SET CURSOR OFF
      nWait := 5                       // Make a reasonalbe delay
      DO CASE
      CASE nKey == K_ENTER             // Initiate edit of date/time
         nWait := 0.1
         _ftShowStatus( cTitle )
         SET CURSOR ON
         READ
         SET CURSOR OFF
      CASE nKey == K_CTRL_ENTER        // Immediate execution
         RESTSCREEN(,,,, cScreen )
         xRetVal := EVAL(bRoutine)
         EXIT
      CASE (DATE() >= dDate) .AND. (TIME() >= cTime)  // Trigger time!
         RESTSCREEN(,,,, cScreen )
         xRetVal := EVAL(bRoutine)
         EXIT
      OTHERWISE                        // Update deferreal status
         _ftShowStatus( cTitle )
         CLEAR GETS
      ENDCASE
   ENDDO

   SETCOLOR( cColor )
   RETURN xRetVal
*  end of FT_DELAYRUN()

******************************* _ftShowStatus() *******************************
STATIC FUNCTION _ftShowStatus( cTitle )
   * Displays current time and execution time at random location on screen
   LOCAL cMsg     := "<ESC:quit  ENTER:edit date/time  CTRL-ENTER:execute>"
   LOCAL nHigh    := 02
   LOCAL nWide    := 2+MAX( LEN(cMsg), LEN(cTitle) )
   /*
   LOCAL nRow     := FT_RAND1(MAXROW()-nHigh)   // Random location, keeping
   LOCAL nCol     := FT_RAND1(MAXCOL()-nWide)   // box entirely within screen
   */
   LOCAL nRow     := _ftRandom(MAXROW()-nHigh)  // Random location, keeping
   LOCAL nCol     := _ftRandom(MAXCOL()-nWide)  // box entirely within screen
   LOCAL cNowIs   := DTOC(DATE())+" "+LEFT(TIME(),5)
   MEMVAR GetList

   CLEAR SCREEN
   @ nRow, nCol++, nRow+nHigh+1, nCol+nWide BOX B_SINGLE+" "

   @ nRow++,nCol     SAY PADC(cTitle, nWide,"")
   @ nRow  ,nCol     SAY "Deferred until date/time .. " GET dDate COLOR "W+/N, N/W"
   @ nRow++,COL()+1  GET cTime PICTURE "@K 99:99" COLOR "W+/N, N/W"
   @ nRow++,nCol     SAY "Current date/time .........  " + cNowis
   @ nRow++,nCol     SAY PADC( cMsg, nWide,"" )

   RETURN NIL
*  end of _ftShowStatus()

********************************* _ftRandom() *********************************
STATIC FUNCTION _ftRandom( nMax )
   STATIC nSeed
   LOCAL m := 100000000, b := 31415621

   nSeed := iif( nSeed == NIL, seconds(), nSeed )   // init_seed()

   RETURN( nMax * ( ( nSeed := mod( nSeed*b+1, m ) ) / m ) )
*  end of _ftRandom()


